\
\ $ProjectVersion: 0.89 $
\
\ Code to drive a silver smartcard (16f876+24lc64)
\
\ This demonstration program does nothing but send the ATR (Answer To Reset)
\ which identifies the card. It has been written to test the EEPROM read
\ routine.

include piceeprom.fs

hex

7 pin-b i/o

\ Timer-start initializes the timer

: timer-start ( -- ) 0 tmr0 ! t0if intcon bit-clr ;

\ A bit correspond to 372/fi where fi is the clock frequency. The following
\ word sets the timer to wait until a timer overflow occurs, then to reset
\ the timer so that the next time it overflows exactly one bit has passed.

: timer-wait ( n -- )
    tmr0 +!
    begin t0if intcon bit-set? until        \ Wait until overflow occurs
    t0if intcon bit-clr                     \ Clear overflow bit
;

: timer-1bit ( -- ) -$5b timer-wait ;       \ 372/4-2=$5b

: timer-400 ( -- ) -$62 timer-wait ;        \ 400/4-2=$62

\ Switch between transmission and reception mode

: transmission-mode ( -- ) i/o high $80 trisb /and! timer-start ;
: reception-mode ( -- ) $80 trisb or! ;

\ Send one or zero bit

variable parity

\ Inverse convention: one is low, zero is high, MSB first

: send-one ( -- ) i/o low 1 parity xor! timer-1bit ;
: send-zero ( -- ) i/o high timer-1bit ;

\ Send byte

variable bitcount

: send-byte ( n -- )
    transmission-mode
    send-one                                 \ Start bit
    0 parity !
    8 bitcount v-for
      rlf-tos                                \ MSB
      c status bit-set? if send-one else send-zero then
    bitcount v-next
    drop
    parity @ if send-one else send-zero then
    reception-mode timer-1bit timer-1bit     \ Mandatory delay between bytes
;

variable tmpee

: send-eeprom ( addr n -- )
    tmpee v-for dup ee@ send-byte 1+ tmpee v-next drop ;

\ Send ATR - This one is used only for test purpose, it has been copied from
\ a French pay-tv card (the only smartcard I had handy when writing this
\ sample code).

eecreate atr $03 ee, $77 ee, $18 ee, $25 ee,
             $00 ee, $29 ee, $00 ee, $14 ee,
             $00 ee, $62 ee, $2d ee, $00 ee,

variable tmpatr
	     
: send-atr ( -- )
    timer-400                          \ Required delay at the beginning
    atr $c send-eeprom
;

\ Initialization

: init ( -- )
    $08 option_reg !                         \ No prescaler, pull-ups enabled
    reception-mode                           \ Default state is reception mode
;

\ Main program

main : main ( -- )
    init
    send-atr
    begin again
;


\ Configuration word

fosc-xt set-fosc      \ Crystal oscillator
false set-wdte        \ Do not use watchdog timer
false set-lvp         \ No low-voltage programming
false set-boden       \ No brown-out reset
false set-wrt         \ Disable flash-writing